{-# LANGUAGE TupleSections #-}
{-# LANGUAGE ScopedTypeVariables #-}

module AutoComplete (auto) where


import Data.List (sortBy)
import Data.List.Split (splitOn)

import qualified Data.ByteString.Char8 as BC
import qualified Data.Trie as T
import qualified Data.Map as M
import qualified Data.Bifunctor as B
import System.Directory
import WordCount

import Control.Exception (Exception, throw)

newtype FormatError = FormatError String
    deriving (Show, Eq)

instance Exception FormatError

loadWords :: FilePath -> IO (M.Map String Int)
loadWords fname = do
    filedata <- readFile fname
    let contents = map (\line -> let ws = splitOn "," line in
                                case ws of
                                    [s, i] -> (s, read i :: Int)
                                    _ -> throw (FormatError line)) (lines filedata)
    let loadedMap = M.fromList contents
    return loadedMap


createTrie :: M.Map String Int -> T.Trie Int
createTrie = T.fromList . fmap (B.first BC.pack) . M.toList

autocomplete :: Ord a => T.Trie a -> IO ()
autocomplete trie = do
  putStrLn "Enter word:"
  prefix <- getLine
  let subTrie = T.submap (BC.pack prefix) trie
      topk = take 10 $ sortBy (\(_, a) (_, b) -> compare b a) $ T.toList subTrie
  if null topk
    then putStrLn "No words found"
    else do
      putStrLn "Found the following words:"
      mapM_ (\(n :: Int, (a, _)) -> putStrLn $ show n ++ ". " ++ BC.unpack a) (zip [1..] topk)
  putStrLn ""
  autocomplete trie


auto :: FilePath -> IO ()
auto directory = do
    outputExists <- doesFileExist ".autocomplete_out"
    if outputExists
        then putStrLn "Output file already exists, skipping wordCount. run with -F to re-read files."
        else wordCount directory ".autocomplete_out"
    loadedMap <- loadWords ".autocomplete_out"
    let trie = createTrie loadedMap
    autocomplete trie
