{-# LANGUAGE TupleSections #-}

module WordCount
( wordCount
)

where

import Data.Char (isAlpha, isSpace, toLower)
import MapReduce
import System.Directory
import qualified Data.Map as M
import System.IO.Error (catchIOError)

mapper :: String -> M.Map String Int
mapper = getWordFreqMap . getWords

getWordFreqMap :: [String] -> M.Map String Int
getWordFreqMap = M.fromListWith (+) . map (, 1)

getWords :: String -> [String]
getWords = words . filter (\x -> isAlpha x || isSpace x) . map toLower

reducer :: [M.Map String Int] -> [(String, Int)]
reducer = M.toList . foldl (M.unionWith (+)) M.empty

wordCount :: FilePath -> FilePath -> IO ()
wordCount filePath output = do
    files <- listDirectory filePath
    let myFiles = filter (\x -> x `notElem` [".", ".."]) (map (filePath ++) files)
    parsedFiles <- mapM readFile myFiles
    let freqs = mapReduce mapper reducer parsedFiles
    printFreqs freqs output


printFreqs :: [(String, Int)] -> FilePath -> IO ()
printFreqs freqs output = do
    let outputText = unlines $ map (\(word, count) -> word ++ "," ++ show count) freqs
    if output == "-"
        then putStrLn outputText
        else catchIOError (writeFile output outputText) (\_ -> putStrLn "Error: could not write to file")
